# ------------------------------------------------------------------------------
# WebDAV for Tclhttpd (webdav_vfs.tcl)
# 
# Support for WebDAV-Access for vfs-mounted files (like tar, zip or starkits)
#
# for further instructions see webdav.tcl
# or 
# http://www.vogel-nest.de (go to "WebDAV")
# 
# HISTORY:
# 2004-07-29: Version 0.1a
#  * initial version
# ------------------------------------------------------------------------------
package require vfs

namespace eval de::vogel::webdav_vfs {
	variable aConfig
	# for the top-level directory-listing only list files
	# with an allowed extension (assuming .kit for starkits)
	array set aConfig {
		lAllowedExtensions {.kit .tar .zip}
	}

	# what is to mounted when a specific file-extension is encountered?
	variable aMountMap 
	array set aMountMap {
		.kit mk4 
		.tar tar
		.zip zip
	}
}


# returns 2-element-list
# path-to-vfs-mount path-inside-mount
# (e.g. /kit/starkit.kit /bin/main.tcl)
proc de::vogel::webdav_vfs::separatePath {path} {
	variable aConfig
	# loop over path and find "vfs"-mount-point
	set i 0
	set lPath [file split $path]
	foreach e $lPath {
		if {[lsearch $aConfig(lAllowedExtensions) [file extension $e]] >= 0} {
			break
		}
		incr i
	}
	if {$i == [llength $lPath]} {
		set i -1
	}
	set l1 [lrange $lPath 0 $i]
	set l2 [lrange $lPath [expr $i+1] end]
	set lResult1 {}
	set lResult2 {}
	if {[llength $l1]} {
		set lResult1 [eval file join $l1]
	}
	if {[llength $l2]} {
		set lResult2 [eval file join $l2]
	}
	return [list $lResult1 $lResult2]
}

proc de::vogel::webdav_vfs::init {prefix} {
	variable aConfig
	set aConfig(prefix) [string trim $prefix /]
}

proc de::vogel::webdav_vfs::checkMountPoint {realPath realVfs} {
	variable aConfig
	variable aMountMap
	if {[catch {vfs::filesystem info $realVfs} err]} {
		# mount this
		if {![info exists aMountMap([file extension $realVfs])]} {
			de::vogel::webdav::log "de::vogel::webdav_vfs::checkMountPoint: Don't know which handler to use for '[file extension $realVfs]'/'$realPath - $realVfs'. Please add this value to the mapping 'aMountMap' in webdav_vfs.tcl!"
			# this doesn't seem to be a vfs_mount, go on as usual
			return
		}
		set namespace $aMountMap([file extension $realVfs])
		if {[catch {
			set hd [vfs::${namespace}::Mount $realVfs $realVfs]
			set aConfig(mountpoints,$realVfs) $hd
		} err]} {
			de::vogel::webdav::log "de::vogel::checkMountPoint: '$realVfs' failed err: $err"
			return 0
		}
		return 1
	} else {
		return 1
	}
}

proc de::vogel::webdav_vfs::PROPFIND {path depth propertiesList} {
	global Config
	variable aConfig
	de::vogel::webdav::log "webdav_vfs::PROPFIND $path"
	if {[string trim $path /] == $aConfig(prefix)} {
		# return [de::vogel::webdav_file::PROPFIND $path $depth $propertiesList]
		set entries [glob -directory $Config(docRoot) -nocomplain [file join $path *]]
		set result ""
		foreach entry $entries {
			if {[lsearch $aConfig(lAllowedExtensions) [file extension $entry]] >= 0} {
				append result [de::vogel::webdav_file::getProp $entry $propertiesList 1]
			}
		}
		set result [de::vogel::webdav::setMultiStatus $result]
		return [list 207 "" text/xml $result data]
	} else {
		foreach {vfs dummy} [de::vogel::webdav_vfs::separatePath $path] {break}
		set realPath [de::vogel::webdav::path2RealPath $path]
		set realVfs [de::vogel::webdav::path2RealPath $vfs]

		de::vogel::webdav_vfs::checkMountPoint $realPath $realVfs
		
		return [de::vogel::webdav_file::PROPFIND $path/ $depth $propertiesList]
	}
}

# never called?
proc de::vogel::webdav_vfs::getProp {path propertiesList} {
	foreach {vfs dummy} [de::vogel::webdav_vfs::separatePath $path] {break}
	set realPath [de::vogel::webdav::path2RealPath $path]
	set realVfs [de::vogel::webdav::path2RealPath $vfs]

	set result [de::vogel::webdav_vfs::checkMountPoint $realPath $realVfs]

	set file [de::vogel::webdav::path2RealPath $path]
}

proc de::vogel::webdav_vfs::GET {path} {
	
	foreach {vfs dummy} [de::vogel::webdav_vfs::separatePath $path] {break}
	set realPath [de::vogel::webdav::path2RealPath $path]
	set realVfs [de::vogel::webdav::path2RealPath $vfs]

	de::vogel::webdav_vfs::checkMountPoint $realPath $realVfs
	return [de::vogel::webdav_file::GET $path]
}

proc de::vogel::webdav_vfs::DELETE {path} {
	variable aConfig
	foreach {dummy locPath} [de::vogel::webdav_vfs::separatePath $path] {break}
	if {$locPath == ""} {
		return [de::vogel::webdav_file::DELETE $path]
	} else {
		# TODO forbidden
	}
}